@@ -31,7 +31,7 @@
  *            Sven Neumann <sven@convergence.de>
  */
 
-#include <config.h>
+#include "config.h"
 #include "gdk.h"
 #include "gdkwindow.h"
 
@@ -646,7 +646,7 @@ gdk_directfb_window_new (GdkWindow              *parent,
 }
 
 GdkWindow *
-gdk_window_new (GdkWindow     *parent,
+_gdk_window_new (GdkWindow     *parent,
                 GdkWindowAttr *attributes,
                 gint           attributes_mask)
 {
@@ -1511,7 +1511,7 @@ gdk_window_move_resize (GdkWindow *window,
 }
 
 void
-gdk_window_reparent (GdkWindow *window,
+_gdk_window_reparent (GdkWindow *window,
                      GdkWindow *new_parent,
                      gint       x,
                      gint       y)
@@ -2684,7 +2684,7 @@ gdk_window_get_frame_extents (GdkWindow    *window,
   impl = GDK_DRAWABLE_IMPL_DIRECTFB (private->impl);
 
   rect->x      = impl->abs_x;
-  rect->y      = impl->abs_x;
+  rect->y      = impl->abs_y;
   rect->width  = impl->width;
   rect->height = impl->height;
 }
